#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* baudrate settings are defined in <asm/termbits.h>, which is
   included by <termios.h> */
#define BAUDRATE B19200           
/* change this definition for the correct port */
#define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */

#define NUMBER 77 /* receiver index no. */

#define FALSE 0
#define TRUE 1

volatile int STOP=FALSE; 
struct termios oldtio,newtio;
int fd;

void ropen(void)
{
  int c, res;
  int number=NUMBER;
  char buf[255];
  /* 
     Open modem device for reading and writing and not as controlling tty
     because we don't want to get killed if linenoise sends CTRL-C.
  */
  fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY ); 
  if (fd <0) {perror(MODEMDEVICE); exit(-1); }
  
  tcgetattr(fd,&oldtio); /* save current serial port settings */
  bzero(&newtio, sizeof(newtio)); /* clear struct for new port settings */
  
  /* 
     BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
     CRTSCTS : output hardware flow control (only used if the cable has
     all necessary lines. See sect. 7 of Serial-HOWTO)
     CS8     : 8n1 (8bit,no parity,1 stopbit)
     CLOCAL  : local connection, no modem contol
     CREAD   : enable receiving characters
  */
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  
  /*
    IGNPAR  : ignore bytes with parity errors
    ICRNL   : map CR to NL (otherwise a CR input on the other computer
    will not terminate input)
    otherwise make device raw (no other input processing)
  */
  newtio.c_iflag = IGNPAR | ICRNL ;
  
  /*
    Raw output.
  */
  newtio.c_oflag = 0;
  
  /*
    ICANON  : enable canonical input
    disable all echo functionality, and don't send signals to calling program
  */
  newtio.c_lflag = ICANON;
  
  /* 
     initialize all control characters 
     default values can be found in /usr/include/termios.h, and are given
     in the comments, but we don't need them here
  */
  newtio.c_cc[VINTR]    = 0;     /* Ctrl-c */ 
  newtio.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
  newtio.c_cc[VERASE]   = 0;     /* del */
  newtio.c_cc[VKILL]    = 0;     /* @ */
  newtio.c_cc[VEOF]     = 4;     /* Ctrl-d */
  newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
  newtio.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */
  newtio.c_cc[VSWTC]    = 0;     /* '\0' */
  newtio.c_cc[VSTART]   = 0;     /* Ctrl-q */ 
  newtio.c_cc[VSTOP]    = 0;     /* Ctrl-s */
  newtio.c_cc[VSUSP]    = 0;     /* Ctrl-z */
  newtio.c_cc[VEOL]     = 0;     /* '\0' */
  newtio.c_cc[VREPRINT] = 0;     /* Ctrl-r */
  newtio.c_cc[VDISCARD] = 0;     /* Ctrl-u */
  newtio.c_cc[VWERASE]  = 0;     /* Ctrl-w */
  newtio.c_cc[VLNEXT]   = 0;     /* Ctrl-v */
  newtio.c_cc[VEOL2]    = 0;     /* '\0' */
  
  /* 
     now clean the modem line and activate the settings for the port
  */
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
  
  /*
    terminal settings done, now handle input
    In this example, inputting a 'z' at the beginning of a line will 
    exit the program.
  */
  //while (STOP==FALSE) {     /* loop until we have a terminating condition */
    /* read blocks program execution until a line terminating character is 
       input, even if more than 255 chars are input. If the number
       of characters read is smaller than the number of chars available,
       subsequent reads will return the remaining chars. res will be set
       to the actual number of characters actually read */

  // select receiver, put in remote mode
  sprintf(buf,"$%2iS2\r",number);
  write(fd,buf,6);
  
}  

  /* restore the old port settings */

void rclose(void){
  // put receiver in local mode and restore serial settings
  write(fd,"S1",2);
  sleep(1);
  tcsetattr(fd,TCSANOW,&oldtio);
}

void getband(float bwtable[7]){
  // read back filter bandwidth table (kHz)

  char buf[255];
  int res;

  write(fd,"S5\r",3);
  res=read(fd,buf,255);
  buf[res]=0;

  bwtable[0]=atof(strtok(buf,","));
  bwtable[1]=atof(strtok(NULL,","));
  bwtable[2]=atof(strtok(NULL,","));
  bwtable[3]=atof(strtok(NULL,","));  
  bwtable[4]=atof(strtok(NULL,","));
  bwtable[5]=atof(strtok(NULL,","));  
  bwtable[6]=atof(strtok(NULL,","));
}

void getfreq(double *freq){
  char buf[255];
  int res;
  write(fd,"TF\r",3);
  res=read(fd,buf,255);
  buf[res]=0; 
  *freq=atof(strtok(buf,"S")+1); // get rid of leading F
}

void getmode(int *mode){
  char buf[255];
  int res;
  write(fd,"TD\r",3);
  res=read(fd,buf,255);
  buf[res]=0; 
  *mode=atoi(strtok(buf,"S")+1); // get rid of leading D
}

void getbw(float *bw){
  char buf[255];
  int res;
  write(fd,"TI\r",3);
  res=read(fd,buf,255);
  buf[res]=0; 
  *bw=atof(strtok(buf,"S")+1); // get rid of leading I
}

void getspeed(int *speed){
  char buf[255];
  int res;
  write(fd,"TM\r",3);
  res=read(fd,buf,255);
  buf[res]=0; 
  *speed=atoi(strtok(buf,"S")+1); // get rid of leading M
}

void getalc(int *alc){
  char buf[255];
  int res;
  write(fd,"TR\r",3);
  res=read(fd,buf,255);
  buf[res]=0; 
  *alc=atoi(strtok(buf,"S")+1); // get rid of leading R
}

void getbfo(float *bfo){
  char buf[255];
  int res;
  write(fd,"TB\r",3);
  res=read(fd,buf,255);
  buf[res]=0; 
  *bfo=atof(strtok(buf,"S")+1); // get rid of leading B
}

void setfreq(double freq){
  char buf[255];
  sprintf(buf,"F%f\r",freq);
  write(fd,buf,strlen(buf));
}

void setmode(int mode){
  char buf[255];
  sprintf(buf,"D%i\r",mode);
  write(fd,buf,strlen(buf));
}

void setspeed(int speed){
  char buf[255];
  sprintf(buf,"M%i\r",speed);
  write(fd,buf,strlen(buf));
}

void setbw(float bw){
  char buf[255];
  sprintf(buf,"I%f\r",bw);
  write(fd,buf,strlen(buf));
}

void setbfo(float bfo){
  char buf[255];
  sprintf(buf,"B%f\r",bfo);
  write(fd,buf,strlen(buf));
}

void setif(int level){
  char buf[255];
  sprintf(buf,"A%i\r",level);
  write(fd,buf,strlen(buf));
} 

void setrem(int remote){
  char buf[255];
  sprintf(buf,"S%i\r",1+remote);
  write(fd,buf,strlen(buf));
}
